@@ -2,6 +2,7 @@ try:
     from __builtin__ import all as _all, any as _any
 except ImportError:
     from builtins import all as _all, any as _any
+from copy import copy
 from operator import itemgetter, methodcaller, attrgetter
 from itertools import chain, tee
 from collections import defaultdict
@@ -19,7 +20,7 @@ __all__ = ['empty', 'iteritems', 'itervalues',
            'is_distinct', 'all', 'any', 'none', 'one', 'some',
            'zipdict', 'flip', 'project', 'omit', 'zip_values', 'zip_dicts',
            'where', 'pluck', 'pluck_attr', 'invoke', 'lwhere', 'lpluck', 'lpluck_attr', 'linvoke',
-           'get_in', 'set_in', 'update_in', 'has_path']
+           'get_in', 'set_in', 'update_in', 'del_in', 'has_path']
 
 
 ### Generic ops
@@ -251,16 +252,16 @@ def omit(mapping, keys):
 def zip_values(*dicts):
     """Yields tuples of corresponding values of several dicts."""
     if len(dicts) < 1:
-        raise TypeError('izip_values expects at least one argument')
+        raise TypeError('zip_values expects at least one argument')
     keys = set.intersection(*map(set, dicts))
     for key in keys:
         yield tuple(d[key] for d in dicts)
 
 def zip_dicts(*dicts):
-    """Yields tuples like (key, val1, val2, ...)
+    """Yields tuples like (key, (val1, val2, ...))
        for each common key in all given dicts."""
     if len(dicts) < 1:
-        raise TypeError('izip_dicts expects at least one argument')
+        raise TypeError('zip_dicts expects at least one argument')
     keys = set.intersection(*map(set, dicts))
     for key in keys:
         yield key, tuple(d[key] for d in dicts)
@@ -293,6 +294,24 @@ def update_in(coll, path, update, default=None):
         copy[path[0]] = update_in(copy.get(path[0], current_default), path[1:], update, default)
         return copy
 
+
+def del_in(coll, path):
+    """Creates a copy of coll with a nested key or index deleted."""
+    if not path:
+        return coll
+    try:
+        next_coll = coll[path[0]]
+    except (KeyError, IndexError):
+        return coll
+
+    coll_copy = copy(coll)
+    if len(path) == 1:
+        del coll_copy[path[0]]
+    else:
+        coll_copy[path[0]] = del_in(next_coll, path[1:])
+    return coll_copy
+
+
 def has_path(coll, path):
     """Checks if path exists in the given nested collection."""
     for p in path:
